#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlcdnumber.h>
#include <qmap.h>
#include <qtimer.h>
#include <qhbox.h>
#include <qvbox.h>

#include <khighscore.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kscoredialog.h>

#include "memcontroller.h"

//========================================
// @CLASS: MemController
//========================================
MemController::MemController(QWidget* parent, int n_match)
  : QWidget(parent)
  , m_layout(new QVBoxLayout(this))
  , m_time_display(0)
  , m_timer(new QTimer(this))
  , m_minutes(0)
  , m_seconds(0)
  , m_n_match(n_match)
  , m_matched(0)
{
  connect(m_timer, SIGNAL(timeout()), this, SLOT(tick()));
}

MemController::~MemController()
{
}

void MemController::init()
{
  m_layout->addItem(new QSpacerItem(5,5, QSizePolicy::Minimum, QSizePolicy::Minimum));

  QHBox* status_hbox = new QHBox(this);
  status_hbox->layout()->setSpacing(5);
  status_hbox->layout()->setMargin(10);
  for (int i=0; i<m_n_match; ++i) {
    QCheckBox* cb = new QCheckBox(status_hbox);
    cb->setEnabled(false);
    m_match_status.push_back(cb);
  }

  m_layout->addWidget(status_hbox);

  m_layout->addWidget(m_time_display = new QLCDNumber(this));

  m_time_display->setNumDigits(5);
  m_time_display->setMinimumHeight(40);
  m_time_display->display("--:--");
}

void MemController::move()
{
  if (!m_timer->isActive()) {
    m_time_display->display("00:00");
    m_timer->start(1000, false);
  }
  m_matched++;
  highlightMatched();
}

void MemController::guess(bool firstSeen)
{
  m_matched = 0;
  highlightMatched();
}

void MemController::miss()
{
  m_matched = 0;
  highlightMatched();
}

void MemController::tick()
{
  m_seconds++;
  if (m_seconds == 60) {
    m_seconds = 0;
    m_minutes++;
  }
  QString mins, secs;
  mins.setNum(m_minutes);
  secs.setNum(m_seconds);
  m_time_display->display(QString("%1:%2")
                          .arg(mins.rightJustify(2, '0'))
                          .arg(secs.rightJustify(2, '0')));
}

void MemController::highlightMatched()
{
  for (int i=0; i<m_match_status.size(); ++i) {
    if (i < m_matched) {
      m_match_status[i]->setChecked(true);
    }
    else {
      m_match_status[i]->setChecked(false);
    }
  }
}

//========================================
// @CLASS: SinglePlayerMemController
//========================================
SinglePlayerMemController::SinglePlayerMemController(QWidget* parent, int num_items, int n_match)
  : MemController(parent, n_match)
  , m_num_items(num_items)
  , m_num_moves(0)
  , m_score(0)
  , m_moves_display(0)
  , m_score_display(0)
{
  m_layout->addWidget(new QLabel("Score", this));
  m_layout->addWidget(m_score_display = new QLCDNumber(this));
  m_layout->addItem(new QSpacerItem(1,5, QSizePolicy::Minimum, QSizePolicy::Minimum ));
  m_layout->addWidget(new QLabel("Moves", this));
  m_layout->addWidget(m_moves_display = new QLCDNumber(this));
  m_layout->addItem(new QSpacerItem(10,2000, QSizePolicy::Minimum, QSizePolicy::Expanding ));

  m_moves_display->setNumDigits(3);
  m_moves_display->setMinimumHeight(60);
  m_moves_display->setMinimumWidth(100);
  m_moves_display->display(0);
  m_score_display->setNumDigits(3);
  m_score_display->setMinimumHeight(60);
  m_score_display->setMinimumWidth(100);
  m_score_display->display(0);

  //setMinimumSize(sizeHint());

  MemController::init();
}

bool SinglePlayerMemController::is_started() const
{
  return m_num_moves != 0;
}

void SinglePlayerMemController::move()
{
  MemController::move();

  m_num_moves++;
  m_moves_display->display(m_num_moves);
}

void SinglePlayerMemController::miss()
{
  MemController::miss();
}

void SinglePlayerMemController::guess(bool firstSeen)
{
  MemController::guess(firstSeen);

  if (firstSeen) {
    m_score += 10;
  }
  else {
    m_score++;
  }
  m_score_display->display(m_score);

  m_matched_items += m_n_match;
  if (m_matched_items == m_num_items) {
    gameOver();
  }
}

void SinglePlayerMemController::gameOver()
{
  KScoreDialog *score_dialog = new KScoreDialog(
                                                KScoreDialog::Name | KScoreDialog::Level | KScoreDialog::Time | KScoreDialog::Score,
                                                this, i18n("High Score"));

  score_dialog->setConfigGroup(QString("Match %1").arg(m_n_match));

  QString mins, secs;
  mins.setNum(m_minutes);
  secs.setNum(m_seconds);

  QMap<int, QString> info;
  info[KScoreDialog::Level] = QString(i18n("Size %1")).arg(m_num_items);
  info[KScoreDialog::Time] = QString("%1:%2")
    .arg(mins.rightJustify(2, '0'))
    .arg(secs.rightJustify(2, '0'));

  score_dialog->addScore(m_score, info);
  score_dialog->show();
}

//========================================
// @CLASS: TwoPlayerMemController
//========================================
TwoPlayerMemController::TwoPlayerMemController(QWidget* parent, int num_items, int n_match)
  : MemController(parent, n_match)
  , m_turn(1)
  , m_num_items(num_items)
  , m_num_moves_1(0)
  , m_num_moves_2(0)
  , m_score_1(0)
  , m_score_2(0)
  , m_score_1_display(0)
  , m_score_2_display(0)
  , m_moves_1_display(0)
  , m_moves_2_display(0)
{
  m_layout->addWidget(new QLabel("Player 1 Score", this));
  m_layout->addWidget(m_score_1_display = new QLCDNumber(this));

  m_layout->addItem(new QSpacerItem(1,5, QSizePolicy::Minimum, QSizePolicy::Minimum ));

  m_layout->addWidget(new QLabel("Player 1 Moves", this));
  m_layout->addWidget(m_moves_1_display = new QLCDNumber(this));

  m_layout->addItem(new QSpacerItem(10,5000, QSizePolicy::Minimum, QSizePolicy::Expanding ));

  m_layout->addWidget(new QLabel("Player 2 Score", this));
  m_layout->addWidget(m_score_2_display = new QLCDNumber(this));

  m_layout->addItem(new QSpacerItem(1,5, QSizePolicy::Minimum, QSizePolicy::Minimum ));

  m_layout->addWidget(new QLabel("Player 2 Moves", this));
  m_layout->addWidget(m_moves_2_display = new QLCDNumber(this));

  m_layout->addItem(new QSpacerItem(1,15, QSizePolicy::Minimum, QSizePolicy::Minimum ));

  m_score_1_display->setNumDigits(3);
  m_score_1_display->setMinimumHeight(60);
  m_score_1_display->setMinimumWidth(100);
  m_score_1_display->display(m_score_1);

  m_score_2_display->setNumDigits(3);
  m_score_2_display->setMinimumHeight(60);
  m_score_2_display->setMinimumWidth(100);
  m_score_2_display->display(m_score_2);

  m_moves_1_display->setNumDigits(3);
  m_moves_1_display->setMinimumHeight(60);
  m_moves_1_display->setMinimumWidth(100);
  m_moves_1_display->display(m_num_moves_1);

  m_moves_2_display->setNumDigits(3);
  m_moves_2_display->setMinimumHeight(60);
  m_moves_2_display->setMinimumWidth(100);
  m_moves_2_display->display(m_num_moves_2);

  highlight_turn();

  MemController::init();
}

bool TwoPlayerMemController::is_started() const
{
  return m_num_moves_1 != 0 || m_num_moves_2 != 0;
}

void TwoPlayerMemController::move()
{
  MemController::move();

  if (m_turn == 1) {
    m_num_moves_1++;
    m_moves_1_display->display(m_num_moves_1);
  }
  else {
    m_num_moves_2++;
    m_moves_2_display->display(m_num_moves_2);
  }
}

void TwoPlayerMemController::miss()
{
  MemController::miss();

  m_turn = 3-m_turn;
  highlight_turn();
}

void TwoPlayerMemController::guess(bool firstSeen)
{
  MemController::guess(firstSeen);

  if (m_turn == 1) {
    m_score_1 += firstSeen ? 10:1;
    m_score_1_display->display(m_score_1);
  }
  else {
    m_score_2 += firstSeen ? 10:1;
    m_score_2_display->display(m_score_2);
  }

  m_matched_items += m_n_match;
  if (m_matched_items == m_num_items) {
    gameOver();
  }

}

void TwoPlayerMemController::gameOver()
{
  if (m_score_1 > m_score_2) {
    KMessageBox::information(this, i18n("Player 1 wins"), i18n("Game Over"));
  }
  else if (m_score_1 < m_score_2) {
    KMessageBox::information(this, i18n("Player 2 wins"), i18n("Game Over"));
  }
  else {
    KMessageBox::information(this, i18n("Friendship wins"), i18n("Game Over"));
  }
}

void TwoPlayerMemController::highlight_turn()
{
  if (m_turn == 1) {
    m_score_1_display->setSegmentStyle(QLCDNumber::Filled);
    m_score_2_display->setSegmentStyle(QLCDNumber::Outline);
  }
  else {
    m_score_1_display->setSegmentStyle(QLCDNumber::Outline);
    m_score_2_display->setSegmentStyle(QLCDNumber::Filled);
  }
}

